<?php
/**
 * Jonah_News:: is the main class for handling news headlines for Jonah both
 * from internal Jonah generated news sources and external channels.
 *
 * $Horde: jonah/lib/News.php,v 1.95 2005/08/24 05:39:13 selsky Exp $
 *
 * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 * Copyright 2004-2005 Jan Schneider <jan@horde.org>
 *
 * See the enclosed file LICENSE for license information (BSD). If you did not
 * receive this file, see http://cvs.horde.org/co.php/jonah/LICENSE.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Marko Djukic <marko@oblo.com>
 * @author  Jan Schneider <jan@horde.org>
 * @package Jonah
 */
class Jonah_News {

    /**
     * Hash containing connection parameters.
     *
     * @var array
     */
    var $_params = array();

    /**
     * Constructs a new News storage object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function Jonah_News($params = array())
    {
        $this->_params = $params;
    }

    function deleteChannel(&$info)
    {
        /* Delete any lists for this channel. */
        require_once JONAH_BASE . '/lib/Delivery.php';
        $delete = Jonah_Delivery::deleteChannelLists($info['channel_id']);
        if (is_a($delete, 'PEAR_Error')) {
            return $delete;
        }

        /* If it is a cached channel delete from cache too. */
        if ($info['channel_type'] = JONAH_EXTERNAL_CHANNEL) {
            require_once 'Horde/Cache.php';
            $cache = &Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'], Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
            if (is_a($cache, 'PEAR_Error')) {
                Horde::fatal($cache, __FILE__, __LINE__);
            }
            // @TODO: call expire on something. URL?
            // $cache->expire();
        }

        /* Delete the channel from the backend. */
        return $this->_deleteChannel($info['channel_id']);
    }

    /**
     * Fetches the requested channel, while actually passing on the request to
     * the backend _getChannel() function to do the real work.
     *
     * @param integer $channel_id  The channel id to fetch.
     *
     * @return array|PEAR_Error  The channel details as an array or a
     *                           PEAR_Error if not valid or not found.
     */
    function getChannel($channel_id)
    {
        static $channel = array();

        /* We need a non empty channel id. */
        if (empty($channel_id)) {
            return PEAR::raiseError(_("Missing channel id."));
        }

        /* Cache the fetching of channels. */
        if (!isset($channel[$channel_id])) {
            $channel[$channel_id] = $this->_getChannel($channel_id);
        }

        return $channel[$channel_id];
    }

    /**
     * Checks if the channel is editable by first checking if the $channel_id
     * returns a valid channel array and then whether the channel type is
     * JONAH_INTERNAL_CHANNEL, which is the only one to allow stories to be
     * added.
     *
     * @param integer $channel_id  The channel id to check.
     *
     * @return array|PEAR_Error  The channel details as an array or a
     *                           PEAR_Error if not editable.
     */
    function isChannelEditable($channel_id)
    {
        /* Check if this channel id returns a valid channel. */
        $channel = $this->getChannel($channel_id);
        if (is_a($channel, 'PEAR_Error')) {
            return $channel;
        }

        /* Check if the channel type allows adding of stories. */
        if ($channel['channel_type'] != JONAH_INTERNAL_CHANNEL) {
            return PEAR::raiseError(sprintf(_("Channel \"%s\" is not an internally authored channel."), $channel['channel_name']));
        }

        return $channel;
    }

    /**
     * Returns the most recent or all stories from a channel.
     *
     * @param integer $channel_id  The news channel to get stories from.
     * @param integer $max         The maximum number of stories to get. If null,
     *                             all stories will be returned.
     * @param integer $from        The number of the story to start with.
     * @param boolean $refresh     Force a refresh of stories in case this is an
     *                             external channel.
     * @param integer $date        The timestamp of the date to start with.
     *
     * @return array  The specified number (or less, if there are fewer) of
     *                stories from the given channel.
     */
    function getStories($channel_id, $max = 10, $from = 0, $refresh = false,
                        $date = 0)
    {
        $channel = $this->getChannel($channel_id);

        /* Fetch the stories according to channel type. */
        switch ($channel['channel_type']) {
        case JONAH_INTERNAL_CHANNEL:
            global $conf, $registry
;
            $channel['channel_link'] = Util::addParameter(Horde::applicationUrl('delivery/html.php', true, -1), 'channel_id', $channel['channel_id']);
            $stories = $this->_getStories($channel_id, $max, $from, $date);
            if (is_a($stories, 'PEAR_Error')) {
                return $stories;
            }
            $date_format = $GLOBALS['prefs']->getValue('date_format');
            if (empty($channel['channel_story_url'])) {
                $url = Horde::applicationUrl('stories/view.php');
                $url = Util::addParameter($url, array('channel_id' => '%c', 'story_id' => '%s'));
                $url = str_replace(array('%25c', '%25s'), array('%c', '%s'), $url);
            } else {
                $url = $channel['channel_story_url'];
            }
            $comments = $conf['comments']['allow'] &&
                $registry->hasMethod('forums/numMessages');
            foreach ($stories as $key => $story) {
                $stories[$key]['story_url'] = str_replace(array('%c', '%s'), array($channel_id, $story['story_id']), $url);
                $stories[$key]['timestamp'] = $story['story_date'];
                $stories[$key]['story_date'] = strftime($date_format, $story['story_date']);
                if ($comments) {
                    $stories[$key]['num_comments'] = $registry->call('forums/numMessages', array($story['story_id'], $registry->getApp()));
                }
                $stories[$key] = array_merge($channel, $stories[$key]);
            }
            break;

        case JONAH_EXTERNAL_CHANNEL:
            if ($refresh) {
                $channel['channel_interval'] = -1;
            }
            $stories = $this->fetchExternalStories($channel_id, $channel['channel_url'], $channel['channel_interval']);
            array_walk($stories, create_function('&$story, $key, $channel', '$story = array_merge($channel, $story);'), $channel);
            if (!is_null($max)) {
                $stories = array_slice($stories, $from, $max);
            }
            break;

        case JONAH_AGGREGATED_CHANNEL:
            if ($refresh) {
                $channel['channel_interval'] = -1;
            }
            $stories = array();
            $date_format = $GLOBALS['prefs']->getValue('date_format');
            $channels = explode(':', $channel['channel_url']);
            foreach ($channels as $id) {
                $channel_data = $this->getChannel($id);
                if (is_a($channel_data, 'PEAR_Error')) {
                    continue;
                }
                $externals = $this->fetchExternalStories(null, $channel_data['channel_url'], $channel['channel_interval']);
                if (is_array($externals)) {
                    foreach ($externals as $external) {
                        /* Check if we have seen this story already. */
                        $story = $this->_getStoryByUrl($channel['channel_id'], $external['story_url']);
                        $info = array();
                        if (is_array($story)) {
                            /* Check if the story is unchanged. */
                            if ($this->getMD5($story) == $this->getMD5($external)) {
                                $story['timestamp'] = $story['story_date'];
                                $story['story_date'] = strftime($date_format, $story['story_date']);
                                $stories[] = array_merge($channel_data, $story);
                                continue;
                            }
                            $info['story_id'] = $story['story_id'];
                        }
                        $info['channel_id'] = $channel['channel_id'];
                        $info['story_title'] = $external['story_title'];
                        $info['story_desc'] = $external['story_desc'];
                        $info['story_url'] = $external['story_url'];
                        $info['story_body'] = isset($external['body']) ? $external['body'] : null;
                        $info['story_body_type'] = 'text';
                        $info['timestamp'] = $external['story_date'];
                        $this->saveStory($info);
                        $info['story_date'] = strftime($date_format, $external['story_date']);
                        $stories[] = array_merge($channel_data, $info);
                    }
                }
            }
            usort($stories, create_function('$a, $b', 'return $b["timestamp"] - $a["timestamp"];'));
            if (!is_null($max)) {
                $stories = array_slice($stories, $from, $max);
            }
            break;

        case JONAH_COMPOSITE_CHANNEL:
            $stories = array();
            $channels = explode(':', $channel['channel_url']);
            foreach ($channels as $subchannel) {
                $stories = array_merge($stories, $this->getStories($subchannel, null, 0, $refresh, $date));
            }
            if (!empty($channel['channel_story_url'])) {
                foreach ($stories as $key => $story) {
                    $stories[$key]['story_url'] = str_replace(array('%c', '%s'), array($story['channel_id'], $story['story_id']), $channel['channel_story_url']);
                }
            }
            usort($stories, create_function('$a, $b', 'return $b["timestamp"] - $a["timestamp"];'));
            if (!is_null($max)) {
                $stories = array_slice($stories, $from, $max);
            }
            break;
        }

        return $stories;
    }

    function saveStory(&$info)
    {
        /* Build release date. */
        if ($info['story_release_now']) {
            $info['story_release'] = mktime($info['story_release_time']['hour'],
                                            $info['story_release_time']['minute'],
                                            0,
                                            date('m'),
                                            date('d'),
                                            date('Y'));
        }

        /* Used for checking whether to send out delivery or not. */
        if (empty($info['story_release'])) {
            /* Story is not being released. */
            $deliver = false;
        } elseif (empty($info['story_id'])) {
            /* Story is new. */
            $deliver = true;
        } else {
            /* Story is old, has it been released already? */
            $oldstory = $this->getStory(null, $info['story_id']);
            $deliver = empty($oldstory['story_release']);
        }

        /* First save to the backend. */
        $save = $this->_saveStory($info);
        if (is_a($save, 'PEAR_Error') || !$deliver) {
            /* Return here also if editing, do not bother doing deliveries for
             * an edited story. */
            return $save;
        }

        /* Check if there are distribution lists for this channel and send out
         * any distributions required. */
        // TODO: don't deliver if no subscribers.
        require_once JONAH_BASE . '/lib/Delivery.php';
        $delivery_drivers = Jonah_Delivery::getDrivers();
        foreach ($delivery_drivers as $driver => $desc) {
            $delivery = &Jonah_Delivery::singleton($driver);
            $delivered = $delivery->singleDeliver($info);
            $driver_info = $delivery->getInfo();
            if (is_a($delivered, 'PEAR_Error')) {
                $GLOBALS['notification']->push(sprintf(_("Could not deliver to the %s distribution list. %s"), $driver_info['name'], $delivered->getMessage()), 'horde.error');
            } elseif ($delivered) {
                $GLOBALS['notification']->push(sprintf(_("Successfully delivered to the %s distribution list."), $driver_info['name']), 'horde.success');
            }
        }
    }

    function getStory($channel_id, $story_id, $read = false)
    {
        if ($channel_id) {
            $channel = $this->getChannel($channel_id);
            if ($channel['channel_type'] == JONAH_EXTERNAL_CHANNEL) {
                return $this->_getExternalStory($channel, $story_id);
            }
        }

        $story = $this->_getStory($story_id, $read);
        if (is_a($story, 'PEAR_Error')) {
            return $story;
        }

        /* If this story should not be marked as read, set story url to
         * view.php so that it gets marked as read when the link gets
         * followed. */
        if (!$read) {
            $url = Horde::applicationUrl('stories/view.php');
            $url = Util::addParameter($url, array('channel_id' => '%c', 'story_id' => '%s'));
            $url = str_replace(array('%25c', '%25s'), array('%c', '%s'), $url);
            $story['story_url'] = str_replace(array('%c', '%s'), array($story['channel_id'], $story['story_id']), $url);
        }

        /* Format the date. */
        $date_format = $GLOBALS['prefs']->getValue('date_format');
        $story['story_date'] = strftime($date_format, $story['story_date']);

        /* Format release date. */
        if ($story['story_release']) {
            $story['story_release_now'] = true;
            $story['story_release_date'] = $story['story_release'];
            $story['story_release_time'] = $story['story_release'];
        }

        return $story;
    }

    function getMD5($story)
    {
        return md5($story['story_title'] . $story['story_desc']);
    }

    /**
     * Fetches a story list from an external channel, caching the data
     * and only actually fetching if there is no valid cache.
     *
     * @param integer $channel_id  The channel id to fetch.
     * @param string  $url         The url from where to fetch the story list.
     * @param integer $interval    The interval in seconds since the last fetch
     *                             after which the cache will be considered
     *                             expired and an actual fetch will be done.
     *
     * @return array  An array of available stories.
     */
    function fetchExternalStories($channel_id, $url, $interval)
    {
        require_once 'Horde/Cache.php';
        $cache = &Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'], Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
        if (is_a($cache, 'PEAR_Error')) {
            Horde::fatal($cache, __FILE__, __LINE__);
        }

        require_once 'Horde/Serialize.php';
        $timestamp = time();
        if ($stories = $cache->get($url, $interval)) {
            $stories = Horde_Serialize::unserialize($stories, SERIALIZE_UTF7_BASIC, NLS::getCharset());
        } else {
            $stories = Jonah_News::_fetchExternalStories($url, $timestamp);
            $cache->set($url, Horde_Serialize::serialize($stories, SERIALIZE_UTF7_BASIC, NLS::getCharset()));
        }
        /* If the stories from cache return the same timestamp as
         * $timestamp it means that the cache has been refreshed. */
        if (!is_null($channel_id)) {
            if ($stories['timestamp'] == $timestamp) {
                $this->_timestampChannel($channel_id, $timestamp);
            }
        }
        unset($stories['timestamp']);

        return $stories;
    }

    function getIntervalLabel($seconds = null)
    {
        $interval = array(1 => _("none"),
                          1800 => _("30 mins"),
                          3600 => _("1 hour"),
                          7200 => _("2 hours"),
                          14400 => _("4 hours"),
                          28800 => _("8 hours"),
                          43200 => _("12 hours"),
                          86400 => _("24 hours"));

        if (is_null($seconds)) {
            return $interval;
        } else {
            return $interval[$seconds];
        }
    }

    /**
     * Returns the available channel types based on what was set in the
     * configuration.
     *
     * @return array  The available news channel types.
     */
    function getAvailableTypes()
    {
        if (isset($types)) {
            return $types;
        }

        static $types = array();

        if (empty($GLOBALS['conf']['news']['enable'])) {
            return $types;
        }
        if (in_array('external', $GLOBALS['conf']['news']['enable'])) {
            $types[JONAH_EXTERNAL_CHANNEL] = _("External Feed");
        }
        if (in_array('internal', $GLOBALS['conf']['news']['enable'])) {
            $types[JONAH_INTERNAL_CHANNEL] = _("Local Channel");
        }
        if (in_array('aggregated', $GLOBALS['conf']['news']['enable'])) {
            $types[JONAH_AGGREGATED_CHANNEL] = _("Aggregated");
        }
        if (in_array('composite', $GLOBALS['conf']['news']['enable'])) {
            $types[JONAH_COMPOSITE_CHANNEL] = _("Composite");
        }

        return $types;
    }

    /**
     * Returns the default channel type based on what was set in the
     * configuration.
     *
     * @return integer  The default news channel type.
     */
    function getDefaultType()
    {
        if (in_array('external', $GLOBALS['conf']['news']['enable'])) {
            return JONAH_EXTERNAL_CHANNEL;
        } else {
            return JONAH_INTERNAL_CHANNEL;
        }
    }

    function getSubscribedChannels()
    {
        return Jonah::prefDecode('channel_list');
    }

    /**
     * Returns the stories of a channel rendered with the specified template.
     *
     * @param integer $channel_id    The news channel to get stories from.
     * @param string  $channel_view  The name of the template to use.
     * @param integer $max           The maximum number of stories to get. If
     *                               null, all stories will be returned.
     * @param integer $from          The number of the story to start with.
     *
     * @return string  The rendered story listing.
     */
    function renderChannel($channel_id, $channel_view, $max = 10, $from = 0)
    {
        $channel = $this->getChannel($channel_id);
        if (is_a($channel, 'PEAR_Error')) {
            return sprintf(_("Error fetching channel. %s"), $channel->getMessage());
        }

        require JONAH_BASE . '/config/templates.php';
        $template = &new Horde_Template();
        $template->set('channel', $channel, true);

        /* Get one story more than requested to see if there are more
         * stories. */
        if (!is_null($max)) {
            $stories = $this->getStories($channel_id, $max + 1, $from, false, time());
        } else {
            $stories = $this->getStories($channel_id, null, 0, false, time());
        }

        if (is_a($stories, 'PEAR_Error')) {
            return $stories->getMessage();
        }

        if (!$stories) {
            $template->set('error', _("No stories are currently available."), true);
            $template->set('stories', false, true);
            $template->set('image', false, true);
            $template->set('form', false, true);
        } else {
            /* Process story summaries. */
            array_walk($stories, create_function('&$value, $key', '$value["story_desc"] = nl2br($value["story_desc"]);'));

            /* Process story links. */
            if ($channel['channel_type'] == JONAH_EXTERNAL_CHANNEL) {
                array_walk($stories, create_function('&$value, $key', 'if (!empty($value["story_body"])) $value["story_url"] = Util::addParameter(Horde::applicationUrl("stories/view.php"), array("channel_id" => $value["channel_id"], "story_id" => $value["story_id"])); else $value["story_url"] = Horde::externalUrl($value["story_url"]);'));
            }

            $template->set('error', false, true);
            $template->set('story_marker', Horde::img('story_marker.png'));
            $template->set('image', false, true);
            $template->set('form', false, true);
            if ($from) {
                $template->set('previous', max(0, $from - $max), false);
            } else {
                $template->set('previous', false, true);
            }
            $more = count($stories) > $max;
            if ($more) {
                $template->set('next', $from + $max);
                array_pop($stories);
            } else {
                $template->set('next', false, true);
            }
            $template->set('stories', $stories, true);
        }

        return String::convertCharset($template->parse($templates[$channel_view]['template']), 'UTF-8');
    }

    /**
     * Returns the provided story as a MIME message.
     *
     * @param array $story  A data array representing a story.
     *
     * @return MIME_Message  The MIME message object containing the story
     *                       parts.
     */
    function &getStoryAsMessage(&$story)
    {
        require_once 'Horde/MIME/Message.php';
        require_once 'Horde/MIME/Headers.php';
        require_once 'Horde/MIME/Part.php';

        /* Create the base MIME message object. */
        $message = &new MIME_Message();

        /* Add the story to the message based on the story's body type. */
        switch ($story['story_body_type']) {
        case 'richtext':
            /* Get a plain text version of a richtext story. */
            $body_html = $story['story_body'];
            require_once 'Horde/Text/Filter.php';
            $body_text = Text_Filter::filter($body_html, 'html2text');

            /* Add description. */
            $body_html = '<p>' . Text_Filter::filter($story['story_desc'], 'text2html', array('parselevel' => TEXT_HTML_MICRO, 'charset' => NLS::getCharset(), 'class' => null, 'callback' => null)) . "</p>\n" . $body_html;
            $body_text = String::wrap('  ' . $story['story_desc'], 70) . "\n\n" . $body_text;

            /* Add the text version of the story to the base message. */
            $message_text = &new MIME_Part('text/plain');
            $message_text->setCharset(NLS::getCharset());
            $message_text->setContents($message_text->replaceEOL($body_text));
            $message_text->setDescription(_("Plaintext Version of Story"));

            /* Add an HTML version of the story to the base message. */
            $message_html = &new MIME_Part('text/html', String::wrap($body_html),
                                           NLS::getCharset(), 'inline');
            $message_html->setDescription(_("HTML Version of Story"));

            /* Add the two parts as multipart/alternative. */
            $basepart = &new MIME_Part('multipart/alternative');
            $basepart->addPart($message_text);
            $basepart->addPart($message_html);

            /* Add $basepart to the message. */
            $message->addPart($basepart);
            break;

        case 'text':
            /* This is just a plain text story. */
            $message_text = &new MIME_Part('text/plain');
            $message_text->setContents($message_text->replaceEOL($story['story_desc'] . "\n\n" . $story['story_body']));
            $message->addPart($message_text);
            break;
        }

        return $message;
    }

    /**
     * Attempts to return a concrete Jonah_News instance based on $driver.
     *
     * @param string $driver  The type of concrete Jonah_News subclass to
     *                        return. The is based on the storage driver
     *                        ($driver). The code is dynamically included.
     *
     * @param array $params   A hash containing any additional configuration or
     *                        connection parameters a subclass might need.
     *
     * @return mixed  The newly created concrete Jonah_News instance, or false
     *                on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['news']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig(array('news', 'storage'), $driver);
        }

        include_once dirname(__FILE__) . '/News/' . $driver . '.php';
        $class = 'Jonah_News_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("No such backend \"%s\" found"), $driver)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Jonah_News instance based
     * on $driver.
     *
     * It will only create a new instance if no Jonah_News instance with the
     * same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Jonah_News::singleton()
     *
     * @param string $driver  The type of concrete Jonah_News subclass to
     *                        return. The is based on the storage driver
     *                        ($driver). The code is dynamically included.
     *
     * @param array $params   A hash containing any additional configuration or
     *                        connection parameters a subclass might need.
     *
     * @return mixed  The created concrete Jonah_News instance, or false on
     *                error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['news']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig(array('news', 'storage'), $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Jonah_News::factory($driver, $params);
        }

        return $instances[$signature];
    }

    /**
     * Get the full body of an external feed story.
     *
     * @access private
     *
     * @param array $channel     The channel the story belongs to.
     * @param integer $story_id  The id of the story in the channel.
     *
     * @return array  The story array.
     */
    function _getExternalStory($channel, $story_id)
    {
        $stories = $this->fetchExternalStories($channel['channel_id'], $channel['channel_url'], $channel['channel_interval']);
        if (isset($stories[$story_id])) {
            return $stories[$story_id];
        }

        return PEAR::raiseError(sprintf(_("Story \"%s\" not found in channel \"%s\"."), $story_id, $channel['channel_title']));
    }

    /**
     * This function is called if cached data isn't available and is what
     * actually does the URL fetching and XML parsing to get the story list.
     * It is only called if the cached data has expired or didn't exist.
     *
     * @access private
     *
     * @param string  $url        The url from where to fetch the story list.
     * @param integer $timestamp  Timestamp of this fetch.
     *
     * @return array  An array of available stories.
     */
    function _fetchExternalStories($url, $timestamp)
    {
        $xml = Jonah::_readURL($url);
        if (is_a($xml, 'PEAR_Error')) {
            Horde::logMessage($xml, __FILE__, __LINE__, PEAR_LOG_ERR);
            return array('timestamp' => $timestamp);
        }

        /* Parse the feed. */
        require_once JONAH_BASE . '/lib/FeedParser.php';
        $parser = &new Jonah_FeedParser();
        if (!$parser->parse($xml)) {
            $GLOBALS['notification']->push(sprintf(_("Error parsing external channel from %s: %s"), $url, $parser->error), 'warning');
        }
        $stories = $parser->structure;
        $parser->cleanup();

        /* Set the passed timestamp. */
        $items = array('timestamp' => $timestamp);
        if (!empty($stories)) {
            $date_format = $GLOBALS['prefs']->getValue('date_format');
            $timestamp = strftime($date_format, $timestamp);
            foreach ($stories['items'] as $key => $story) {
                $items[$key]['story_id'] = $key;
                $items[$key]['story_title'] = isset($story['title']) ? $story['title'] : _("[No title]");
                $items[$key]['story_desc'] = isset($story['description']) ? $story['description'] : '';
                $items[$key]['story_body'] = isset($story['body']) ? $story['body'] : null;
                $items[$key]['story_url'] = isset($story['link']) ? $story['link'] : '';
                $items[$key]['story_date'] = $timestamp;
            }
        }

        return $items;
    }

}
